
import java.util.*;
import ant.*;

public class BinaryParticlePosition implements StateMachineBlueprint {
    
    /**
     *  Position of particle as array bits
     */
    private BitSet positionVector_;
    public BitSet positionVector() { return positionVector_; }
    public void positionVector( BitSet aBitSet ) { positionVector_ = aBitSet; }

    /**
     *  Count to support the Enumeration interface
     */
    private int enumerationCount_;
    private int enumerationCount() { return enumerationCount_; }
    private void enumerationCount( int anInt ) { enumerationCount_ = anInt; }
    
    /**
     *  The dimension of this ParticlePositon
     */
    private int dimension_;
    public int dimension() { return dimension_; }
    public void dimension( int anInt ) { dimension_ = anInt; }
    
    public static BinaryParticlePosition fromString(String theString) {
        
        int len = theString.length();
        BinaryParticlePosition thePosition = 
                                new BinaryParticlePosition(len);
        BitSet theBitSet = new BitSet(len);
        
        for (int i=0; i<len; i++) {
            if (theString.charAt(i) == '1') {
                theBitSet.set(i);
            }
            else {
                theBitSet.clear(i);
            }
        }
        thePosition.positionVector(theBitSet);
        return thePosition;
    }
        
    public BinaryParticlePosition() {
        this.initialize();
    }
    
    public BinaryParticlePosition(int dimension) {
        this();
        this.dimension(dimension);
        this.positionVector(new BitSet(this.dimension()));
        this.setToZero();
    }
    
    private void initialize() {
        this.dimension(2);
        this.positionVector(new BitSet(this.dimension()));
        this.setToZero();
    }
    
    public void setToZero() {
        for (int i=0; i<this.dimension(); i++) {
            this.valueAt(i, 0);
        }
    }
    
    public int valueAt(int position) {
        if (this.positionVector().get(position))
            return 1;
        return 0;
    }
    
    public void valueAt(int position, int value) {
        if (value==1) 
            this.positionVector().set(position);
        else
            this.positionVector().clear(position);
    }
    
    public Enumeration elements() {
        this.enumerationCount(0);
        return (Enumeration) this;
    }
    
    public boolean hasMoreElements() {
        return (this.enumerationCount() < this.dimension());
    }
    
    public int nextElement() {
        int v = this.valueAt(this.enumerationCount());
        this.enumerationCount(this.enumerationCount()+1);
        return v;
    }
    
    public void setPositionToPositionOf(BinaryParticlePosition p) {
        for (int i=0; i<this.dimension(); i++) {
            this.valueAt(i, p.valueAt(i));
        }
    }
    
    public String toString() {
        StringBuffer s = new StringBuffer();
        for (int i=0; i<this.dimension(); i++) {
            s.append (this.valueAt(i));
        }
        return s.toString();
    }
    
/*    
    public ParticlePosition incrementBy (ParticlePosition p) {
        for (int i=0; i<this.dimension(); i++) {
            this.valueAt(i, this.valueAt(i) + p.valueAt(i));
        }
        return this;
    }
    
    public ParticlePosition divideBy (ParticlePosition p) {
        for (int i=0; i<this.dimension(); i++) {
            this.valueAt(i, this.valueAt(i) / p.valueAt(i));
        }
        return this;
    }
  
    public ParticlePosition scaleBy (double v) {
        for (int i=0; i<this.dimension(); i++) {
            this.valueAt(i, this.valueAt(i) * v);
        }
        return this;
    }
    
*/

// StateMachineBlueprint interface methods

/*
    public String toString() {
        StringBuffer s = new StringBuffer(this.length());
        for (int i=0; i<this.length(); i++) {
            s.append (this.valueAt(i));
        }

        return s.reverse().toString();
    }
*/
        
    /*        
    public StateMachineBlueprint fragment(int start, int end) {  // inclusive
        BinaryParticlePosition c = new BinaryParticlePosition((end - start + 1));
        int j=0;
        for (int i=start; i<=end; i++) {
            c.valueAt(j, this.valueAt(i));
            j++;
        }
        return (StateMachineBlueprint) c;
    }
*/    
    public int length() {
        return this.dimension();
    }
    
    public int fragmentAsInteger(int begin, int end) {
        // converts fragment of a BitSet into an int
        // begin and end are inclusive
        int positionValue = 1;
        int value = 0;
        for (int i=end; i>=begin; i--) {
            if (this.positionVector().get(i)) {
                value += positionValue;
            }
            positionValue *= 2;
        }
        return value;
    }
    
    public int asInteger() {
        int positionValue = 1;
        int value = 0;
        for (int i=this.length()-1; i>=0; i--) {
            if (this.valueAt(i)==1) {
                value += positionValue;
            }
            positionValue *= 2;
        }
        return value;
    }
    
    public BitSet bitSet() {
        return this.positionVector();
    }

    
}
