
import java.util.*;
import ant.*;

public class BinaryParticleSpace extends Observable {
        
    /**
     *  The number of dimensions in this ParticleSpace
     */
    private int dimension_;
    public int dimension() { return dimension_; }
    public void dimension( int anInt ) { dimension_ = anInt; }
    
    /**
     *  Where the Particles live
     */
    private ParticleRepository particleRepository_;
    public ParticleRepository particleRepository() { return particleRepository_; }
    public void particleRepository( ParticleRepository aParticleRepository ) { particleRepository_ = aParticleRepository; }

    /** this is the new way for evaluating fitness. rather than rewrite this class
     * for each problem, we use a different object in this variable that implelents
     * BinaryPSOFitnessEvaluator. The exact class name comes from the config file
     * so we can run the same PSO engine on any number of wildly different problems 
     */
    private BinaryPSOFitnessEvaluator fitnessEvaluator_;
    public BinaryPSOFitnessEvaluator fitnessEvaluator() { return fitnessEvaluator_; } 
    public void fitnessEvaluator( BinaryPSOFitnessEvaluator x ) { fitnessEvaluator_ = x; } 

    /**
     *  Used to determine the fitness of the 'programs' created
     *  by the particles for the Ants to execute on the FoodPlain
     */
    //private AntFSMTraverser antFSMTraverser_;
    //public AntFSMTraverser antFSMTraverser() { return antFSMTraverser_; }
    //public void antFSMTraverser( AntFSMTraverser anAntFSMTraverser ) { antFSMTraverser_ = anAntFSMTraverser; }

    public BinaryParticleSpace(int dimension, int population) {
        this();
        this.dimension(dimension);
        this.createParticles(population);
    }
    
    private BinaryParticleSpace() {
        this.initialize();
    }
    
    private void initialize() {
        this.dimension(2);
        this.particleRepository(new ParticleRepository());
        //this.antFSMTraverser(new AntFSMTraverser());
        
        try
        {
            String className = Configuration.instance().valueOf( "FITNESS_EVALUATOR_CLASS" );
            System.out.println("className = " + className);
            Class c = Class.forName( className );
            System.out.println("class = " + c);
            Object o = c.newInstance();
            System.out.println("o = " + o);
            
            BinaryPSOFitnessEvaluator b = (BinaryPSOFitnessEvaluator) o;
            System.out.println("b = " + b);
            this.fitnessEvaluator( b );
            
        //  this.fitnessEvaluator((BinaryPSOFitnessEvaluator) (Class.forName( Configuration.instance().valueOf("FITNESS_EVALUATOR_CLASS") ).newInstance()));
        }
        catch( Exception e )
        {
            System.out.println("exception during classForName trying to instantiate the fitness evaluator from config file, e = " + e);
        }
         
        ObservableRegistry.instance().register("particle space", this);
    }
    
    public BinaryParticlePosition nullPosition() {
        BinaryParticlePosition p = new BinaryParticlePosition(this.dimension());
        for (int i=0; i<p.dimension(); i++) {
            p.valueAt(i, 0);
        }        
        return p;
    }
    
    public BinaryParticlePosition randomPosition() {
        
        BinaryParticlePosition p = new BinaryParticlePosition(this.dimension());
        for (int i=0; i<p.dimension(); i++) {
            if (Randomizer.instance().next() > 0.5)
                p.valueAt(i, 1);
            else
                p.valueAt(i, 0);
        }
        return p;        
    }
    
    public BinaryParticlePredisposition nullPredisposition() {
        BinaryParticlePredisposition v = new BinaryParticlePredisposition(this.dimension());
        for (int i=0; i<v.dimension(); i++) {
            v.valueAt(i, 0.0);
        }
        return v;        
    }
    
    public BinaryParticlePredisposition randomPredisposition() {
        BinaryParticlePredisposition v = new BinaryParticlePredisposition(this.dimension());
        for (int i=0; i<v.dimension(); i++) {
            v.valueAt(i, ((Randomizer.instance().next() * 
                     2.0 * this.maxParticlePredisposition()) - this.maxParticlePredisposition() ));
        }
        return v;        
    }
 
    public void createParticles(int particleCount) {
    
        // make the particles
        
        for (int i=0; i<particleCount; i++)  {
            
            BinaryParticlePosition       p = this.randomPosition();
            BinaryParticlePredisposition v = this.randomPredisposition();
            
            this.particleRepository().addParticle(new Particle(p,v));        
        }
        
        // now introduce them to their social group
        
        this.createSocialGroups();
        
    }

    
    public void createSocialGroups() {
        
        // (k=2)
        
        Enumeration theElts = this.elements();

        Particle theFirstElement = (Particle) theElts.nextElement();
        Particle thePreviousElement = theFirstElement;
        Particle theCurrentElement = null;

        while (theElts.hasMoreElements()) {
            theCurrentElement = (Particle) theElts.nextElement();
            theCurrentElement.companionA(thePreviousElement);
            thePreviousElement.companionB(theCurrentElement);
            thePreviousElement = theCurrentElement;
        }

        // now make the closed loop

        theCurrentElement.companionB(theFirstElement);
        theFirstElement.companionA(theCurrentElement);
    }
    
    public double maxParticlePredisposition() {
       return Configuration.instance().doubleValueOf("MAX_PARTICLE_PREDISPOSITION");
    }
    
    public Vector particles() {
        return this.particleRepository().particles();
    }
    
    public Enumeration elements() {
        return this.particles().elements();
    }
    
    public double distanceBetween(Particle p1, Particle p2) {
        
        // do not know optimum strategy ..
        // for right now the Hamming distance
        
        double sum = 0.0;
        BinaryParticlePosition pos1 = p1.currentPosition();
        BinaryParticlePosition pos2 = p2.currentPosition();
        
        for (int i=0; i<this.dimension(); i++) {
            int a = pos1.valueAt(i);
            int b = pos2.valueAt(i);
            int c = a + b;
            if ((c==0) || (c==2)) sum = sum + 1.0;
        }
        
        return sum;
    }
    
    public double diversity() {
        Vector v = this.particles();
        int particleCount = v.size();
        double sum = 0.0;
        for (int i=0; i<particleCount; i++) {
            for (int j=i+1; j<particleCount; j++) {
                Particle p1 = (Particle) v.elementAt(i);
                Particle p2 = (Particle) v.elementAt(j);
                double d = this.distanceBetween(p1, p2);
                sum = sum + d;
            }
        }
        return sum / this.dimension();
    }
                
     public Vector diversities() { // fixbg should be part of ProblemSpace
        Vector out = new Vector();
        Enumeration theOC = this.elements();
        while (theOC.hasMoreElements()) {
            Particle aParticle = (Particle) theOC.nextElement();
            double d = 89-this.fitnessFor(aParticle.bestPositionSoFar());
            out.addElement(new Double(d));
        }
        return out;
    }
    
    public double unskewedFitnessFor(BinaryParticlePosition position) {
        return this.fitnessEvaluator().evaluateFitnessFor(position);
    }
        
    public double fitnessFor(BinaryParticlePosition position) {
        
        double fitness = this.unskewedFitnessFor(position);
        int skew = Configuration.instance().intValueOf("RANDOM_FITNESS_SKEW");        
        int mod = 0;
        double r = Randomizer.instance().next();
        
        if (r >= 0.6666) mod =   skew;;
        if (r <= 0.3333) mod = - skew;
        
        fitness = fitness + mod;
        
        return fitness;  // zero is best fitness
    }
        
    public BinaryParticlePosition firstParticlePosition() {
        Enumeration theOC = this.particles().elements();
        Particle p = (Particle) theOC.nextElement();
        this.publicize(p.bestPositionSoFar());
        return p.bestPositionSoFar();
    }
    
    public synchronized BinaryParticlePosition bestGlobalPositionSoFar() {
        
        double f = Double.POSITIVE_INFINITY;
        BinaryParticlePosition pos = this.nullPosition();
        
        // find best position discovered so far
        
        Vector particles = this.particles();
        Enumeration theOC = particles.elements();
        
        while(theOC.hasMoreElements()) {
            Particle p = (Particle) theOC.nextElement();
            
            if ( this.fitnessFor(p.bestPositionSoFar()) < f) {
                f = this.fitnessFor(p.bestPositionSoFar());
                pos = p.bestPositionSoFar();
            }
        }
        this.publicize(pos);
        return pos;
    }
    
    public Vector descriptionOf (BinaryParticlePosition position) {
        return this.fitnessEvaluator().descriptionOf(position);
    }
    
    public void publicize(BinaryParticlePosition p) {
        this.setChanged();
        this.notifyObservers(p);
    }    
                
}
