import java.util.*;
//import java.awt.Point;
//import torroidialfield.*;
import java.awt.Color;

public class BinaryParticleSwarmCustodian {

    /**
     *  Where the particles work
     */
    private BinaryParticleSpace particleSpace_;
    public BinaryParticleSpace particleSpace() { return particleSpace_; }
    public void particleSpace( BinaryParticleSpace aParticleSpace ) { particleSpace_ = aParticleSpace; }

    /**
     *  The number of computation steps performed on the
     *  Particles in the associated ParticleSpace
     */
    private int currentStepCount_;
    public int currentStepCount() { return currentStepCount_; }
    public void currentStepCount( int anInt ) { currentStepCount_ = anInt; }
        
    /**
     *  True if someone has requested this custodian to
     *  shut down. The inspector checks this and will shut
     *  down when it is activated.
     */
    private boolean shutDownIsRequested_;
    public boolean shutDownIsRequested() { return shutDownIsRequested_; }
    public void shutDownIsRequested( boolean aBoolean ) { shutDownIsRequested_ = aBoolean; }

        
    public BinaryParticleSwarmCustodian() {
        this.initialize();
    }
    
    private void initialize() {
        this.shutDownIsRequested(false);
        this.particleSpace(new BinaryParticleSpace(
                 Configuration.instance().intValueOf("PROBLEM_DIMENSION"),
                     Configuration.instance().intValueOf("INITIAL_POPULATION")));
    }
               
    public Vector allActors() {
        Vector actors = new Vector();
        actors.addAll(this.particleSpace().particles());
        return actors;
    }
    
    public synchronized double particleDiversity() {
        return this.particleSpace().diversity();
    }
    
    public synchronized Vector particleDiversities() {
        return this.particleSpace().diversities();
    }
    
    public synchronized void advanceParticlesOneTimeStep() {
    
        Particle bestNeighbor = null;
    
        Enumeration theOC = this.particleSpace().elements();
        while (theOC.hasMoreElements()) {
            Particle theParticle = (Particle) theOC.nextElement();
            
            // make sure Particle saves best solution
            
            double currentFitness   = this.evaluateFitnessForPosition(
                                             theParticle.currentPosition());
            double bestFitnessSoFar = this.evaluateFitnessForPosition(
                                             theParticle.bestPositionSoFar());
                                             
            if (currentFitness < bestFitnessSoFar) { // was <= fixbg
                theParticle.bestPositionSoFar().
                       setPositionToPositionOf(theParticle.currentPosition());
            }
        }
            
        // now loop through the particles again
        
        theOC = this.particleSpace().elements();
        while (theOC.hasMoreElements()) {
            Particle theParticle = (Particle) theOC.nextElement();
            
            // first find the best neighbor
            
            double fitnessForA = this.evaluateFitnessForPosition(
                                      theParticle.companionA().bestPositionSoFar());
            double fitnessForB = this.evaluateFitnessForPosition(
                                      theParticle.companionB().bestPositionSoFar());
            
            if (fitnessForA < fitnessForB) {
                bestNeighbor =  theParticle.companionA();
            }
            else  {
                bestNeighbor =  theParticle.companionB();
            }
            
            // now update the solution and veloity for the
            // current particle based on its and its 
            // neighbors performance
            
            this.updatePredispositionFor (theParticle, bestNeighbor);
            this.updatePositionFor (theParticle);            
        }
        
    }
    
    public synchronized void reset() {
        this.currentStepCount(0);
        // this.particleSpace().createTheParticles();
    }
    
    public synchronized void nextStep() {
        this.currentStepCount(this.currentStepCount() + 1);
        this.advanceParticlesOneTimeStep();                        
    }
    
    
    public double socialWeightingFactor()  {
        return (Randomizer.instance().next() * 
          Configuration.instance().doubleValueOf("PHIMAX"));
    }
    
    public double evaluateFitnessForPosition(BinaryParticlePosition position) {
        return this.particleSpace().fitnessFor(position);
    }
    
    public double evaluateUnskewedFitnessForPosition(BinaryParticlePosition position) {
        return this.particleSpace().unskewedFitnessFor(position);
    }
    
    public synchronized void updatePredispositionFor (Particle theParticle,
                                        Particle theBestNeighbor)  {
                                        
        double phi = this.socialWeightingFactor();
        double phiMax = Configuration.instance().doubleValueOf("PHIMAX");
        
                                        
    
        BinaryParticlePredisposition predisposition = theParticle.currentPredisposition();
        BinaryParticlePosition bestLocalPosition    = theParticle.bestPositionSoFar();
        BinaryParticlePosition bestGlobalPosition   = theBestNeighbor.bestPositionSoFar();
        BinaryParticlePosition currentPosition      = theParticle.currentPosition();
        
        for (int i=0; i<this.particleSpace().dimension(); i++) {
    
            double newPredisposition = predisposition.valueAt(i) + (phiMax - phi) * 
                                                   (bestLocalPosition.valueAt(i) -
                                                 currentPosition.valueAt(i)) + phi *
                                                       (bestGlobalPosition.valueAt(i) - 
                                                              currentPosition.valueAt(i));
                                        
            if (newPredisposition >  this.particleSpace().maxParticlePredisposition() )
                newPredisposition =  this.particleSpace().maxParticlePredisposition();
            if (newPredisposition < -this.particleSpace().maxParticlePredisposition()) 
                newPredisposition = -this.particleSpace().maxParticlePredisposition();
        
            theParticle.currentPredisposition().valueAt(i, newPredisposition); 
        }
    }
    
    public synchronized void updatePositionFor (Particle theParticle)  {
        
        for (int i=0; i<this.particleSpace().dimension(); i++) { 
            double rho = Randomizer.instance().next();
            if (rho < this.sigmoidOf(theParticle.currentPredisposition().valueAt(i)))
                theParticle.currentPosition().valueAt(i, 1);
            else
                theParticle.currentPosition().valueAt(i, 0);
        }
    }
    
    public double sigmoidOf(double v) {
        return (1.0 / (1 + Math.exp(-v)));
    } 
    
    public synchronized void displayAll() {
        this.displayParticlePositions();
    }
    
    public synchronized void displayParticlePositions() {
        int j = 0;
        System.out.println();
        Enumeration theOC = this.particleSpace().elements();
        while (theOC.hasMoreElements()) {
            System.out.print("P" + j + " ");
            Particle aParticle = (Particle) theOC.nextElement();
            BinaryParticlePosition p = aParticle.currentPosition();
            for (int i=0; i<this.particleSpace().dimension(); i++) {
                System.out.print(p.valueAt(i));
            }
            System.out.println();
            j++;
        }
    }
    
    public synchronized BinaryParticlePosition bestGlobalPositionSoFar() {
        return this.particleSpace().bestGlobalPositionSoFar();
    }
    
    public synchronized double bestGlobalFitnessSoFar() {
        return this.evaluateFitnessForPosition(this.bestGlobalPositionSoFar());
    }
    
    public synchronized double bestGlobalUnskewedFitnessSoFar() {
        return this.evaluateUnskewedFitnessForPosition(this.bestGlobalPositionSoFar());
    }
    
    public synchronized void computeAveragePosition() {
        //this.particleSpace().computeAveragePosition();
    }
    
    public String logInfo() {
        return ("" + (89-this.bestGlobalUnskewedFitnessSoFar()));
    }
      
}    
