import java.util.*;
import graph.*;

public class BinaryParticleSwarmInspector extends Observable implements Runnable {
    
    /**
     *  The custodian this ParticleSwarmInspector has
     *  been sent to inspect
     */
    private BinaryParticleSwarmCustodian particleSwarmCustodian_;
    public BinaryParticleSwarmCustodian particleSwarmCustodian() { return particleSwarmCustodian_; }
    public void particleSwarmCustodian( BinaryParticleSwarmCustodian aParticleSwarmCustodian ) { particleSwarmCustodian_ = aParticleSwarmCustodian; }

    /**
     *  Particle diversity graph
     */
    public LineGraph diversityGraph_;
    private LineGraph diversityGraph() { return diversityGraph_; }
    private void diversityGraph( LineGraph aGraph ) { diversityGraph_ = aGraph; }
    
    /**
     *  Shows Particle diversity as Histogram
     */
    private Histogram diversityHistogram_;
    public Histogram diversityHistogram() { return diversityHistogram_; }
    public void diversityHistogram( Histogram anHistogram ) { diversityHistogram_ = anHistogram; }

    /**
     *  Indicates this Inspector is busy
     */
    private boolean busy_;
    public boolean busy() { return busy_ ; }
    public void busy(boolean aBoolean) { busy_ = aBoolean ; }
    
    /**
     *  Request inspections to stop at the next breaking point
     */
    private boolean stopIsRequested_;
    public boolean stopIsRequested() { return stopIsRequested_; }
    public void stopIsRequested(boolean aBoolean) { stopIsRequested_ = aBoolean ; }
    
    /**
     *  The Thread associated with this Inspector
     */
    private Thread thread_;
    public Thread thread() { return thread_ ; }
    public void thread(Thread aThread) { thread_ = aThread ; }


    private BinaryParticleSwarmInspector() {
        this.initialize();
    }
    
    public BinaryParticleSwarmInspector(BinaryParticleSwarmCustodian theCustodian) {
        this();
        this.particleSwarmCustodian(theCustodian);
    }
    
    private void initialize() {
        this.thread(new Thread(this));
        this.busy(false);
        this.stopIsRequested(false);
        
        this.diversityGraph(new LineGraph(new Vector(), 10, 1, false, false));
        ObservableRegistry.instance().register("linegraph", this.diversityGraph());
         
        this.diversityHistogram(new Histogram(20, 2, 90, false, true));
        ObservableRegistry.instance().register("histogram", this.diversityHistogram());
        
        ObservableRegistry.instance().register("inspector", this);
        this.thread().start();
    }
    
    public void publicize(Double d) { // publish most food found so far
        this.setChanged();
        this.notifyObservers(d);
    }
    
    public int evaluationCycleCount() {
        return this.particleSwarmCustodian().currentStepCount();
    }
    
    public void stopAllThreads() {
        this.stopIsRequested(true);
    }
            
    public void run() {
        
        double d = 0.0;
    
        System.out.println("Particle Swarm Inspector thread starting");
    
        this.stopIsRequested(false);
    
        while (!this.stopIsRequested()) {
                                    
            if (d < 1.0e-10) d = 1.0e-10;
            //d = Math.log(d); // display the log
            this.diversityGraph().addSequentialPoint(d);
            this.diversityGraph().plot();
            
            if (Configuration.instance().booleanValueOf("SHOWDIVERSITY")) {
 
                Vector v = this.particleSwarmCustodian().particleDiversities();
                Enumeration theOC = v.elements();
                this.diversityHistogram().reset();
                while(theOC.hasMoreElements()) {
                    d = ((Double) theOC.nextElement()).doubleValue();
                    if (d < 1.0e-10) d = 1.0e-10;
                    //d = Math.log(d); // display the log
                    this.diversityHistogram().addDataValue(d);
                }
                this.diversityHistogram().plot();
            } 
                        
            // check if shut down is requested
            // if so let the view know at next publicize
            if (this.particleSwarmCustodian().shutDownIsRequested()) {
                System.out.println("Inspector shutting down");
                this.stopIsRequested(true);
            }
            
            d = 89-this.particleSwarmCustodian().bestGlobalFitnessSoFar();
            this.publicize(new Double(d));
            
            try  {
                Thread.sleep(Configuration.instance().
                       intValueOf("INSPECTIONINTERVAL")); // inter inspection peroid
            }
            catch (InterruptedException e)  {
            }
            
        }
        
        System.out.println("Done - ");
    }    
}
