import java.util.*;
import java.awt.*;
import java.awt.event.*;
import graph.*;


public class BinaryParticleSwarmInspectorView extends SCView {
    
    /**
     *  The inspector associated with this View
     */
    private BinaryParticleSwarmInspector binaryParticleSwarmInspector_;
    public BinaryParticleSwarmInspector binaryParticleSwarmInspector() { return binaryParticleSwarmInspector_; }
    public void binaryParticleSwarmInspector( BinaryParticleSwarmInspector aBinaryParticleSwarmInspector ) { binaryParticleSwarmInspector_ = aBinaryParticleSwarmInspector; }


    private BinaryParticleSwarmInspectorView() {
        super("ParticleSpace Inspector");
        this.initialize();
    }
    
    public BinaryParticleSwarmInspectorView(ParticleSwarmSimulatorView superView,
                                      BinaryParticleSwarmCustodian custodianToInspect) {
        super("Particle Swarm Inspector");
        superView.addSubView("inspector", this);
        this.binaryParticleSwarmInspector(new BinaryParticleSwarmInspector(custodianToInspect));
        this.initialize();
        this.show();
    }
    
    private void initialize() { 
        //this.particleSwarmSimulator(new ParticleSwarmSimulator()); // force creation of main model
        this.setLayout(null);
        this.setSize(550,700);
        this.setLocation(200,30);
        this.setResizable(false);
        this.initMenus();
        this.initLists();
        this.initLabels();
        this.initCanvasses();
        this.hookupObservers();
    }
    
    public void hookupObservers() {
        (ObservableRegistry.instance().
             obtainObservable("inspector")).addObserver(this);                       
    }  
    
    public void initCanvasses() {
        
        // average position
        
        GraphCanvas theGraphCanvas = new LineGraphCanvas("Best Meal So Far");
        theGraphCanvas.setSize(513,160);
        theGraphCanvas.setLocation(20, 100);
        theGraphCanvas.initializeGraphics();
        
        (ObservableRegistry.instance().
             obtainObservable("linegraph")).addObserver(theGraphCanvas); 
               
        this.addWidget("graph canvas", theGraphCanvas);
        
        // diversity histogram
        
        HistogramCanvas theHistogramCanvas = new HistogramCanvas("Fitness Histogram");
        theHistogramCanvas.setSize(513,160);
        theHistogramCanvas.setLocation(20, 300);
        theHistogramCanvas.initializeGraphics();
        
        (ObservableRegistry.instance().
             obtainObservable("histogram")).addObserver(theHistogramCanvas); 
             
        this.addWidget("histogram canvas", theHistogramCanvas);
    
        /*

        TorroidialFieldCanvas theCanvas = new TopologicalFieldCanvas();
        //TorroidialFieldCanvas theCanvas = new TorroidialFieldCanvas();
        theCanvas.setSize(Configuration.instance().intValueOf("CANVASWIDTH"),
                              Configuration.instance().intValueOf("CANVASHEIGHT"));
        theCanvas.setLocation(30,60);
        ObservableRegistry.instance().obtainObservable("fieldone").addObserver(theCanvas);
        this.addWidget("field canvas", theCanvas);
        */
    } 
    
    public void initLabels() {
        Label aLabel = new Label("Most food found so far: ");
        //aLabel.setFont(new Font("Courier", Font.PLAIN, 14));
        aLabel.setLocation(30, 55);
        aLabel.setSize(140,20);
        //this.setForeground(new Color(Configuration.instance().hexValueOf("TEXTCOLOR")));
        this.add(aLabel);
        
        aLabel = new Label("                 ");
        //aLabel.setFont(new Font("Courier", Font.PLAIN, 14));
        aLabel.setLocation(180, 55);
        aLabel.setSize(60,20);
        //this.setForeground(new Color(Configuration.instance().hexValueOf("TEXTCOLOR")));
        this.addWidget("mostFoodFoundSoFar", aLabel);
        
        aLabel = new Label("Current Step Count:");
        //aLabel.setFont(new Font("Courier", Font.PLAIN, 14));
        aLabel.setLocation(300, 55);
        aLabel.setSize(180,20);
        //this.setForeground(new Color(Configuration.instance().hexValueOf("TEXTCOLOR")));
        this.add(aLabel);

        aLabel = new Label("                 ");
        //aLabel.setFont(new Font("Courier", Font.PLAIN, 14));
        aLabel.setLocation(500, 55);
        aLabel.setSize(60,20);
        //this.setForeground(new Color(Configuration.instance().hexValueOf("TEXTCOLOR")));
        this.addWidget("currentStepCount", aLabel);
    }
    
    public void initLists() {
        this.addWidget("best particle info", new BinaryParticlePositionList(this));
        //this.addWidget("chromosome program list", new ChromosomeProgramList(this));
    }

    public void initMenus() {
        
        MenuBar aMenuBar = new MenuBar();
        
            Menu aMenu = new Menu("File");
                MenuItem aMenuItem = new MenuItem("New");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("New");
                aMenu.add(aMenuItem);
                
                aMenuItem = new MenuItem("Exit");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("Exit");
                aMenu.add(aMenuItem);
            aMenuBar.add(aMenu);
                
            aMenu = new Menu("Edit");
                aMenuItem = new MenuItem("Copy");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("Copy");
                aMenu.add(aMenuItem);
                
                aMenuItem = new MenuItem("Paste");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("Paste");
                aMenu.add(aMenuItem);
            aMenuBar.add(aMenu);
            
            aMenu = new Menu("Action");
                aMenuItem = new MenuItem("Advance One Timestep");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("Advance One Timestep");
                aMenu.add(aMenuItem);
                
                aMenuItem = new MenuItem("Advance Continuously");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("Advance Continuously");
                aMenu.add(aMenuItem);
                
                aMenuItem = new MenuItem("Stop");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("Stop");
                aMenu.add(aMenuItem);
            aMenuBar.add(aMenu);
            
            aMenu = new Menu("View");
                aMenuItem = new MenuItem("Particles");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("Particles");
                aMenu.add(aMenuItem);
                
                aMenuItem = new MenuItem("Statistics");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("Statistics");
                aMenu.add(aMenuItem);
                
                aMenuItem = new MenuItem("-");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("separator");
                aMenu.add(aMenuItem);
                
            aMenuBar.add(aMenu);

        this.setMenuBar(aMenuBar);
    }
    
   public void actionPerformed(ActionEvent e){

        String actionCommand = e.getActionCommand();
        if( actionCommand.equals("New") ) {
            //this.setupEvolutionaryParameters();
            return;
        }
        if( actionCommand.equals("Exit") ){
            this.exit();
            return;
        }
        if( actionCommand.equals("Copy") ){
            this.copyToClipboard();
            return;
        }
        if( actionCommand.equals("Paste") ){
            this.pasteFromClipboard();
            return;
        }
        if( actionCommand.equals("Advance One Timestep") ){
            //this.evolveAGeneration();
            return;
        }
        if( actionCommand.equals("Advance Continuously") ){
            //this.continuousEvolution();
            return;
        }
        if( actionCommand.equals("Stop") ){
            //this.stopEvolution();
            return;
        }
        if( actionCommand.equals("Particles") ){
            //this.spawnPopulationViewer();
            return;
        }
        if( actionCommand.equals("Statistics") ){
            //this.displayStatistics();
            return;
        }
   }
   
   // methods associated with Menus
   
    public void copyToClipboard() {
        System.out.println("copying to clipboard");
    }
    
    public void pasteFromClipboard() {
    }

    public void windowClosing(WindowEvent we){
        this.exit();
    }

    private void exit(){
        this.dispose();
        //System.exit(0);
    }
    
    // normal methods
    
    public void stopAllThreads() {
        this.binaryParticleSwarmInspector().stopAllThreads();
    }
    
    public void update (Observable obs, Object o) {
        BinaryParticleSwarmInspector theInspector = (BinaryParticleSwarmInspector) obs;
        ((Label) this.getWidget("mostFoodFoundSoFar")).
                            setText(((Double) o).toString());
        ((Label) this.getWidget("currentStepCount")).
                            setText(""+theInspector.evaluationCycleCount());
        if (theInspector.stopIsRequested()) {
            System.out.println("Inspector View shutting down");
            this.dispose();
        }
    }
    
     
}