
import java.lang.*;
import java.util.*;
import java.io.*;

public class Configuration {

    private static Configuration instance_ = null;
    private static String FILE = "config.ini";

    private Properties properties_;
    public Properties properties() { return properties_; }
    public void properties (Properties theProp) { properties_ = theProp; }

    private Configuration() {
        this.initialize();
    }

    public synchronized static Configuration instance() {
        if (instance_ == null)
            instance_ = new Configuration();
        return instance_;
    }

    private void initialize() {
        this.properties (new Properties());
        this.load();
        this.display();
    }

    public String valueOf(String aKey) {
        String answer = this.properties().getProperty(aKey);
        if (answer == null) {
            System.out.println ("Config data " + aKey + " not found");
        }
        return answer;
    }
    
    public void updateValue(String theKey, String theValue) {
        this.properties().put(theKey, theValue);
        this.rewrite();
    }
    
    public void updateIntValue(String theKey, int theValue) {
        this.properties().put(theKey, String.valueOf(theValue));
        this.rewrite();
        return;
    }
    
    public void updateDoubleValue(String theKey, double theValue) {
        this.properties().put(theKey, String.valueOf(theValue));
        this.rewrite();
        return;
    }
    
     public void updateHexValue(String theKey, int theValue) {
        this.properties().put(theKey, Integer.toString(theValue, 16));
        this.rewrite();
        return;
    }
    
    public void updateBooleanValue(String theKey, boolean theValue) {
        this.properties().put(theKey, String.valueOf(theValue));
        this.rewrite();
        return;
    }
   
    public int intValueOf(String aKey) {
        int theValue = (new Integer(this.valueOf(aKey))).intValue();
        return theValue;
    }
    
    public double doubleValueOf(String aKey) {
        double theValue = (new Double(this.valueOf(aKey))).doubleValue();
        return theValue;
    }
    
    public int hexValueOf(String aKey) {
        return Integer.parseInt(this.valueOf(aKey), 16);
    }
    
    public boolean booleanValueOf(String aKey) {
        return Boolean.valueOf(this.valueOf(aKey)).booleanValue();
    }

    public void load() {
        try {
            DataInputStream theStreamIn = new DataInputStream (
                    new FileInputStream (Configuration.FILE));
            this.properties().load(theStreamIn);
        }
        catch (IOException e) {
            System.out.println("INI FileIn Error, reloading defaults");
            this.writeDefaults();
        }
    }
    
    public void rewrite() {
        try {
            OutputStream theStreamOut = new FileOutputStream (Configuration.FILE);
            //this.properties().store(theStreamOut, "Artificial Ant Settings");
            this.properties().store(theStreamOut, "Particle Swarm Settings");
        }
        catch (IOException e) { System.out.println("INI FileOut Error"); }
    }

    public void writeDefaults() {

        Properties theSettings = this.properties();
        theSettings.put("RANDOMSEED", "12345");
        theSettings.put("CANVASWIDTH", "700");
        theSettings.put("CANVASHEIGHT", "700");
        theSettings.put("GRIDWIDTH", "32");
        theSettings.put("GRIDHEIGHT", "32");
        theSettings.put("FITNESS_EVALUATOR_CLASS", "ant.AntFSMTraverser");
        theSettings.put("INSPECTIONINTERVAL", "4000");
        theSettings.put("SHOWDIVERSITY", "false");
        theSettings.put("INITIAL_POPULATION", "500");
        theSettings.put("MAX_PARTICLE_PREDISPOSITION", "40.0");
        theSettings.put("PHIMAX", "4.0");
        theSettings.put("PROBLEM_DIMENSION", "256");
        theSettings.put("SOCIAL", "true");
        theSettings.put("RANDOM_FITNESS_SKEW", "0");
        theSettings.put("MAX_GENERATIONS_PER_TRIAL", "9999999999");
        theSettings.put("MAX_TRIALS", "9999999999");
        theSettings.put("DEBUG", "on");

        try {
            OutputStream theStreamOut = new FileOutputStream (Configuration.FILE);
            //theSettings.store(theStreamOut, "Artificial Ant Settings");
            theSettings.store(theStreamOut, "Particle Swarm Settings");
        }
        catch (IOException e) { System.out.println("INI FileOut Error"); }

    }

    public void display() {
        Enumeration theOC = this.properties().propertyNames();
        while (theOC.hasMoreElements()) {
            String key = (String) theOC.nextElement();
            System.out.println (key + " value is " +
                           this.properties().getProperty(key));
        }
    }
    
    public void displayOn(PrintStream p) {
        Enumeration theOC = this.properties().propertyNames();
        while (theOC.hasMoreElements()) {
            String key = (String) theOC.nextElement();
            p.println (key + " value is " +
                           this.properties().getProperty(key));
        }
    }    

}