
import java.util.*;
import java.io.*;

public class ParticleSwarmSimulator extends Observable implements Runnable, DataLoggerClient {


    /**
     *  The custodian for the ParticleSpace
     */
    private BinaryParticleSwarmCustodian particleSwarmCustodian_;
    public BinaryParticleSwarmCustodian particleSwarmCustodian() { return particleSwarmCustodian_; }
    public void particleSwarmCustodian( BinaryParticleSwarmCustodian aParticleSwarmCustodian ) { particleSwarmCustodian_ = aParticleSwarmCustodian; }

    /**
     *  Write data to disk for logging of Particle progress
     */
    private DataLogger dataLogger_;
    public DataLogger dataLogger() { return dataLogger_; }
    public void dataLogger( DataLogger aDataLogger ) { dataLogger_ = aDataLogger; }

    /**
     *  Indicates this Simulator is busy
     */
    private boolean busy_;
    public boolean busy() { return busy_ ; }
    public void busy(boolean aBoolean) { busy_ = aBoolean ; }
    
    /**
     *  Request actors to stop at the next breaking point
     */
    private boolean stopIsRequested_;
    public boolean stopIsRequested() { return stopIsRequested_; }
    public void stopIsRequested(boolean aBoolean) { stopIsRequested_ = aBoolean ; }
    
    /**
     *  The Thread associated with this Simulator
     */
    private Thread thread_;
    public Thread thread() { return thread_ ; }
    public void thread(Thread aThread) { thread_ = aThread ; }
        
    public ParticleSwarmSimulator() {
        this.initialize();
    }
    
    private void initialize() {
        this.particleSwarmCustodian(new BinaryParticleSwarmCustodian());
        this.thread(new Thread(this));
        this.busy(false);
        this.stopIsRequested(false);
        this.dataLogger(new DataLogger(this));
    }
    
    public void reset() {
        this.particleSwarmCustodian(new BinaryParticleSwarmCustodian());
        this.thread(new Thread(this));
        this.busy(false);
        this.stopIsRequested(false);
    }
    
    
    public void startTheEngine() {
        System.out.println("Particle Swarm Simulator starting ..");
        this.thread().start(); // start the work
    }
    
    public void stopAllThreads() {
        this.stopIsRequested(true);
    }
                        
    public void run() {
    
        System.out.println("Particle Swarm Simulator thread starting");
        
        //for (int i=0; i<Configuration.instance().intValueOf("MAX_TRIALS"); i++) {
    
        int count = Configuration.instance().intValueOf("MAX_GENERATIONS_PER_TRIAL");
        this.stopIsRequested(false);
    
        //long startTime = System.currentTimeMillis();
        //int q = 0;
        
        //this.particleSwarmCustodian().reset();
        
        // a new custodian should be made each time
        
        try  {
            Thread.sleep(100); //fixbg .. will not work without this
        }
        catch (InterruptedException e)  {
        }
        
        while (!this.stopIsRequested()) {
            
            try  {
                Thread.sleep(1); // be a good citizen
            }
            catch (InterruptedException e)  {
            }
            
            this.particleSwarmCustodian().nextStep();
            
            count = count - 1;
            if (count <= 0) this.stopIsRequested(true);
        }
        
        // write the data
        this.dataLogger().writeToLog(this.particleSwarmCustodian().logInfo());
        
        // close the model and inspector
        //this.particleSwarmCustodian().shutDownIsRequested(true);;
    
        //this.reset();
        
        this.publicize(); // tell everyone we are done with a trial
    }
    
    public void publicize() {
        this.setChanged();
        this.notifyObservers();
    }
    
    // DataLogger interface methods
    
    public int loggingWriteInterval() {
        return 4;
    }
    
    public boolean includeTimestamp() {
        return true;
    }
    
    public void logHeaderToPrintStream(PrintStream p) {
        Configuration.instance().displayOn(p);        
    }
    
    
}
