package ant;

import java.util.*;
import java.awt.Point;
import Configuration;

public class AntBody {
    
    /**
     *  The direction this ant is facing .. 0-> north, 90->east ..
     */
    private int direction_;
    public int direction() { return direction_; }
    public void direction( int anInt ) { direction_ = anInt; }

    /**
     *  The amount of food this ant has consumed in a run
     */
    private int foodEaten_;
    public int foodEaten() { return foodEaten_; }
    public void foodEaten( int anInt ) { foodEaten_ = anInt; }

    /**
     *  The position of this ant on the FoodPlain
     */
    private Point position_;
    public Point position() { return position_; }
    public void position( Point aPoint ) { position_ = aPoint; }
    
    /**
     *  The number of instructions the current ant
     *  has processed since a reset.
     */
    private int executionCount_;
    public int executionCount() { return executionCount_; }
    public void executionCount( int anInt ) { executionCount_ = anInt; }

    public AntBody() {
        this.initialize();
    }
    
    private void initialize() {
        this.reset();
    }
    
    public void reset() {
        this.direction(90);
        this.position(new Point(0,1)); // starting position
        this.foodEaten(0);
        this.executionCount(0);
    }
    
    public void incrementExecutionCount() {
        this.executionCount(this.executionCount() + 1);
    }
    
    // actions our little buddy can perform ..
    
    public void moveForwardOn(FoodPlain theFoodPlain) {
        this.position(this.nextForwardPosition());
        // do I eat?
        if (theFoodPlain.hasFoodAt(this.position())) {
            theFoodPlain.removeFoodAt(this.position());
            this.foodEaten(this.foodEaten()+1);
        }
        this.incrementExecutionCount();
    }
    
    public void faceLeft() {
        this.direction((this.direction() + 270) % 360);
        this.incrementExecutionCount();
    }
    
    public void faceRight() {
        this.direction((this.direction() + 90) % 360);
        this.incrementExecutionCount();
    }
        
    public void nop() {
        this.incrementExecutionCount();
    }
    
    public void stop() {
        this.incrementExecutionCount();
    }
    
    // support routines
    
    // this was an instruction, but is not under the FSM    
    public boolean ifFoodAheadOn(FoodPlain theFoodPlain) {
        return (theFoodPlain.hasFoodAt(this.nextForwardPosition()));            
    }
    
    public Point nextForwardPosition() {
        
        int gridWidth  = Configuration.instance().intValueOf("GRIDWIDTH");
        int gridHeight = Configuration.instance().intValueOf("GRIDHEIGHT");
        
        // dumb but fast ...
        
        Point oldPosition = this.position();
        Point newPosition = new Point(0,0);
        
        if (this.direction() == 0) {
            newPosition.x = oldPosition.x;
            if (oldPosition.y == 0) {
                newPosition.y = gridHeight-1;
            }
            else {
                newPosition.y = oldPosition.y -1;
            }
            return newPosition;
        }
        
        if (this.direction() == 90) {
            newPosition.y = oldPosition.y;
            if (oldPosition.x == gridWidth-1) {
                newPosition.x = 0;
            }
            else {
                newPosition.x = oldPosition.x + 1;
            }
            return newPosition;
        }
        
        if (this.direction() == 180) {
            newPosition.x = oldPosition.x;
            if (oldPosition.y == gridHeight-1) {
                newPosition.y = 0;
            }
            else {
                newPosition.y = oldPosition.y + 1;
            }
            return newPosition;
        }
        
        if (this.direction() == 270) {
            newPosition.y = oldPosition.y;
            if (oldPosition.x == 0) {
                newPosition.x = gridWidth-1;
            }
            else {
                newPosition.x = oldPosition.x - 1;
            }
            return newPosition;
        }
        return newPosition; // to satisfy the compiler 
    }
}