package antview;

import java.util.*;
import java.awt.event.*;
import java.awt.*;

import graph.*;

import ant.AntFSMTraverser;
import ant.StateMachineBlueprint;
import ObservableRegistry;
import Configuration;
import SCView;

public class AntFSMTraverserView extends SCView implements Runnable {
    
    /**
     *  The AntFSMTraverser for this AntFSMTraverserView
     */
    private AntFSMTraverser antFSMTraverser_;
    public AntFSMTraverser antFSMTraverser() { return antFSMTraverser_; }
    public void antFSMTraverser( AntFSMTraverser anAntFSMTraverser ) { antFSMTraverser_ = anAntFSMTraverser; }
    
        
    private AntFSMTraverserView() {
        super("Ant Tryouts");
        this.initialize();
    }
    
    public AntFSMTraverserView(String title) {
        super(title);
        this.initialize();
    }
    
    public AntFSMTraverserView(String title, StateMachineBlueprint blueprint) {
        // this constructor is a one shot .. the ant runs around then it's done
        super(title);
        this.initialize();
        ((FoodPlainPanel) this.getWidget("foodPlainPanel")).allowUpdate(true); // show the ant
        this.antFSMTraverser().loadStateMachineBlueprint(blueprint);
        //this.antFSMTraverser().execute();
        Thread thread = new Thread(this);
        thread.start();
    }
    
    private void initialize() {

        this.antFSMTraverser(new AntFSMTraverser());
        this.setLayout(null);
        //this.setSize(1024,768);
        this.setSize(550,550);
        this.setLocation(1,1);
        this.setBackground(Color.black);
        this.initPanels();
        this.initCanvasses();
        this.initMenus();
        this.initLists();
        this.initLabels();
        this.hookupObservers();
        this.show();
    }
    
    public void hookupObservers() {
        
        this.antFSMTraverser().foodPlain().addObserver((Observer) this.getWidget("foodPlainPanel"));
        
        //Observable obs = ObservableRegistry.instance().obtainObservable("foodPlain");
        //obs.addObserver((Observer) this.getWidget("foodPlainPanel"));
   }
    
    public void initLabels() {
        Label aLabel = new Label("            ----Fitness----");
        aLabel.setFont(new Font("Courier", Font.PLAIN, 14));
        aLabel.setLocation(30, 55);
        aLabel.setSize(420,20);
        //this.setForeground(new Color(Configuration.instance().hexValueOf("TEXTCOLOR")));
        this.add(aLabel);
    }
        
    public void initPanels() {
        this.addWidget("foodPlainPanel", new FoodPlainPanel());
    }
    
    public void initCanvasses() {
        
    }
    
    public void initLists() {
    }
    
    
    public void initMenus() {
        
        MenuBar aMenuBar = new MenuBar();
        
            Menu aMenu = new Menu("File");
                MenuItem aMenuItem = new MenuItem("New");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("New");
                aMenu.add(aMenuItem);
                
                aMenuItem = new MenuItem("Exit");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("Exit");
                aMenu.add(aMenuItem);
            aMenuBar.add(aMenu);
                
            aMenu = new Menu("Edit");
                aMenuItem = new MenuItem("Copy");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("Copy");
                aMenu.add(aMenuItem);
                
                aMenuItem = new MenuItem("Paste");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("Paste");
                aMenu.add(aMenuItem);
            aMenuBar.add(aMenu);
            
            aMenu = new Menu("Action");
                aMenuItem = new MenuItem("Evolve a Generation");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("Evolve a Generation");
                aMenu.add(aMenuItem);
                
                aMenuItem = new MenuItem("Continuous Evolution");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("Continuous Evolution");
                aMenu.add(aMenuItem);
                
                aMenuItem = new MenuItem("Stop Evolution");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("Stop Evolution");
                aMenu.add(aMenuItem);
            aMenuBar.add(aMenu);
            
            aMenu = new Menu("View");
                aMenuItem = new MenuItem("Population");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("Population");
                aMenu.add(aMenuItem);
                
                aMenuItem = new MenuItem("Statistics");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("Statistics");
                aMenu.add(aMenuItem);
                
                aMenuItem = new MenuItem("-");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("separator");
                aMenu.add(aMenuItem);
                
                aMenuItem = new MenuItem("View Working Ant");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("View Working Ant");
                aMenu.add(aMenuItem);
                
                aMenuItem = new MenuItem("Hide Working Ant");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("Hide Working Ant");
                aMenu.add(aMenuItem);
            aMenuBar.add(aMenu);

        this.setMenuBar(aMenuBar);
    }
    
   public void actionPerformed(ActionEvent e){

        String actionCommand = e.getActionCommand();
        if( actionCommand.equals("New") ) {
            //this.setupEvolutionaryParameters();
            return;
        }
        if( actionCommand.equals("Exit") ){
            this.exit();
            return;
        }
        if( actionCommand.equals("Copy") ){
            this.copyToClipboard();
            return;
        }
        if( actionCommand.equals("Paste") ){
            this.pasteFromClipboard();
            return;
        }
        if( actionCommand.equals("Evolve a Generation") ){
            //this.evolveAGeneration();
            return;
        }
        if( actionCommand.equals("Continuous Evolution") ){
            //this.continuousEvolution();
            return;
        }
        if( actionCommand.equals("Stop Evolution") ){
            //this.stopEvolution();
            return;
        }
        if( actionCommand.equals("Population") ){
            //this.spawnPopulationViewer();
            return;
        }
        if( actionCommand.equals("Statistics") ){
            //this.displayStatistics();
            return;
        }
       if( actionCommand.equals("View Working Ant") ){
            //this.allowFoodCanvasUpdates();
            return;
        }
        if( actionCommand.equals("Hide Working Ant") ){
            //this.disallowFoodCanvasUpdates();
            return;
        }
   }
   
   
   // methods associated with menus
    
    public void copyToClipboard() {
        System.out.println("copying to clipboard");
    }
    
    public void pasteFromClipboard() {
    }
    
    
    public void windowClosing(WindowEvent we){
        this.exit();
    }

    private void exit(){
        this.dispose();
        //System.exit(0);
    }
    
    public void run() {
        System.out.println("Execution Limit = " + this.antFSMTraverser().executionLimit());
        this.antFSMTraverser().execute();
    }

}