package antview;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import ant.FoodPlain;
import Configuration;
import ant.AntBody;

public class FoodPlainPanel extends Panel implements Observer {
    
    private static final int BACKGROUNDCOLOR = 0xff000000;
    private static final int TEXTCOLOR       = 0xffaa5500;
    /**
     *  The current FoodPlain associated with this FoodPlainPanel
     */
    private FoodPlain currentFoodPlain_;
    public FoodPlain currentFoodPlain() { return currentFoodPlain_; }
    public void currentFoodPlain( FoodPlain aFoodPlain ) { currentFoodPlain_ = aFoodPlain; }

    /**
     *  The food canvasses associated with this FoodPlainPanel
     */
    private Vector foodCanvasses_;
    public Vector foodCanvasses() { return foodCanvasses_; }
    public void foodCanvasses( Vector aVector ) { foodCanvasses_ = aVector; }

    /**
     *  The canvas that moves around the FoodPlainPanel
     *  and faces different directions
     */
    private AntCanvas antCanvas_;
    public AntCanvas antCanvas() { return antCanvas_; }
    public void antCanvas( AntCanvas anAntCanvas ) { antCanvas_ = anAntCanvas; }

    /**
     *  The size of the FoodPlain grid .. the number of
     *  vertical and horizontal squares .. not the pixels
     */
    private Point gridSize_;
    public Point gridSize() { return gridSize_; }
    public void gridSize( Point aPoint ) { gridSize_ = aPoint; }

    /**
     *  The size (in pixels) of a grid cell
     */
    private Point cellSize_;
    public Point cellSize() { return cellSize_; }
    public void cellSize( Point aPoint ) { cellSize_ = aPoint; }
    
    /**
     *  Controls whether publicizes from the FoodCanvas will be
     *  displayed.
     */

    private boolean updateIsAllowed_;
    public boolean updateIsAllowed() { return updateIsAllowed_; }
    public void updateIsAllowed( boolean aBoolean ) { updateIsAllowed_ = aBoolean; }
    
    /**
     *  Reflects the amount of food consumed by the current
     *  ant
     */
    public Label foodEatenLabel_;
    private Label foodEatenLabel() { return foodEatenLabel_; }
    private void foodEatenLabel( Label aLabel ) { foodEatenLabel_ = aLabel; }

    /**
     *  Reflects the number of operations performed by the current
     *  ant
     */
    public Label executionCountLabel_;
    private Label executionCountLabel() { return executionCountLabel_; }
    private void executionCountLabel( Label aLabel ) { executionCountLabel_ = aLabel; }
    
    
    public FoodPlainPanel() {
        this.initialize();
    }
    
    private void initialize() {
        
        this.setLayout(null);
        this.setSize(513, 513);
        this.setLocation(20, 40);
        this.setForeground(Color.black);
        //this.setForeground(Color.gray);
        this.setBackground(Color.black);
        //this.setBackground(new Color(FoodPlainPanel.BACKGROUNDCOLOR));
        
        this.currentFoodPlain(new FoodPlain());
        this.updateIsAllowed(false) ; 
            
        this.gridSize( new Point(Configuration.instance().intValueOf("GRIDWIDTH"),
                                    Configuration.instance().intValueOf("GRIDHEIGHT")));
        this.cellSize(new Point(16, 16));
        this.initFoodCanvases();
        this.initAntCanvas();
        this.initLabels();
   }
        
    public void initAntCanvas() {
        this.antCanvas(new AntCanvas(new Point(50, 50)));
        this.add(this.antCanvas());
        this.antCanvas().setVisible(false); // not appropriate to show him until he
                                            // gets a body and a chromosome
        //this.antCanvas().updateAntPositionAndDirection(new Point(0, 1), 180);
    }
    
    public void initFoodCanvases() {
        /* fixbg .. 
        I am not crazy about this because it counts on a dummy FoodCanvas
        to get the food locations.. I think it would be better to make
        a FoodElement object and have the FoodPlain have a collection of
        these .. but this would slow things down a bit since to find if
        there was food ahead the ant would have to tick through the collection
        of food elements or do a hash table lookup .. both slower than the
        current array lookup .. oh well mabye a cool solution will present
        itself ..
        */
        int horizontalCells = this.gridSize().x;
        int verticalCells   = this.gridSize().y;
        
        int cellWidth = this.cellSize().x;
        int cellHeight = this.cellSize().y;
        
        this.foodCanvasses(new Vector());
        
        for (int i=0; i<horizontalCells; i++) {
            for (int j=0; j<verticalCells; j++) {
                if (this.currentFoodPlain().grid()[i][j] == FoodPlain.FOOD) {
                    FoodCanvas theFoodCanvas = new FoodCanvas(new Point(i, j), 
                                          new Point(cellWidth*i+1, cellHeight*j+1));
                    this.foodCanvasses().addElement(theFoodCanvas);
                    theFoodCanvas.setFoodImageRandomly();
                    this.add(theFoodCanvas);
                }
            }
        }
    }
    
    public void initLabels() {
        Label aLabel = new Label("    ");
        aLabel.setFont(new Font("Courier", Font.PLAIN, 14));
        aLabel.setLocation(490, 1);
        aLabel.setSize(40,40);
        aLabel.setForeground(new Color(FoodPlainPanel.TEXTCOLOR));
        aLabel.setBackground(new Color(FoodPlainPanel.BACKGROUNDCOLOR));
        this.add(aLabel);
        this.foodEatenLabel(aLabel);
        
        aLabel = new Label("    ");
        aLabel.setFont(new Font("Courier", Font.PLAIN, 14));
        aLabel.setLocation(450, 1);
        aLabel.setSize(40,40);
        aLabel.setForeground(new Color(FoodPlainPanel.TEXTCOLOR));
        aLabel.setBackground(new Color(FoodPlainPanel.BACKGROUNDCOLOR));
        this.add(aLabel);
        this.executionCountLabel(aLabel);
    }
                
    public void allowUpdate(boolean isAllowed) {
        this.updateIsAllowed(isAllowed);
    }
        
    public void update(Observable obs, Object o){
        if(this.updateIsAllowed()) {
            try{Thread.sleep(80);}catch(InterruptedException e){}
            this.currentFoodPlain((FoodPlain) obs);
            AntBody theAnt = (AntBody) o;
            this.updateFoodCanvassesVisibility((FoodPlain) obs);
            this.updateAntCanvasPosition(theAnt.position(), theAnt.direction());
            
            // late change 
            this.foodEatenLabel().setText("" + theAnt.foodEaten());
            this.executionCountLabel().setText("" + theAnt.executionCount());
            
            //this.repaint();
        }
    }
    
    public void updateFoodCanvassesVisibility(FoodPlain theFoodPlain) {
        
        Enumeration theOC = this.foodCanvasses().elements();
        byte[][] theGrid = this.currentFoodPlain().grid();
        while (theOC.hasMoreElements()) {
            FoodCanvas aFoodCanvas = (FoodCanvas) theOC.nextElement();
            if (theGrid[aFoodCanvas.gridLocation().x]
                                     [aFoodCanvas.gridLocation().y] == FoodPlain.FOOD) {
                aFoodCanvas.makeVisible();
            }
            else {
                aFoodCanvas.makeInvisible();
            }
        }
    }
    
    public void updateAntCanvasPosition(Point gridLocation, int direction) {
        int cellWidth = this.cellSize().x;
        int cellHeight = this.cellSize().y;
        this.antCanvas().updateAntPositionAndDirection(
                                 new Point(cellWidth*gridLocation.x+1,
                                            cellHeight*gridLocation.y+1), direction);
    }
    
    public void paint (Graphics g) {
        
        // draw the grid
        
        int cellWidth = this.cellSize().x;
        int cellHeight = this.cellSize().y;
        int horizontalCells = this.gridSize().x;
        int verticalCells   = this.gridSize().y;
        
        // verticals
        for (int i=0; i<horizontalCells+1; i++) {
            //g.drawLine(i*cellWidth, 0, i, cellHeight*verticalCells);
            g.drawLine(i*cellWidth, 0, i*cellWidth, cellHeight*verticalCells);
        }
        
        // horizontals
        for (int j=0; j<verticalCells+1; j++) {
            //g.drawLine(0, j*cellHeight, cellWidth*horizontalCells, j);
            g.drawLine(0, j*cellHeight, cellWidth*horizontalCells, j*cellHeight);
        }
    }
    
    public void update(Graphics g) {  paint(g);  }
}