package graph;

import java.awt.*;
import java.util.*;
import java.lang.*;
import java.io.*;
import ObservableRegistry;

public abstract class Graph extends Observable {

    private Point graphSize_;
    public Point graphSize() { return graphSize_; }
    public void graphSize(Point theSize) { graphSize_ = theSize; }

    private Vector dataSeries_;
    public Vector dataSeries() { return dataSeries_; }
    protected void dataSeries(Vector theData) { dataSeries_ = theData; }

    private double minX_;
    public double minX() { return minX_; }
    public void minX(double aDouble) { minX_ = aDouble; }

    private double maxX_;
    public double maxX() { return maxX_; }
    public void maxX(double aDouble) { maxX_ = aDouble; }

    private double minY_;
    public double minY() { return minY_; }
    public void minY(double aDouble) { minY_ = aDouble; }

    private double maxY_;
    public double maxY() { return maxY_; }
    public void maxY(double aDouble) { maxY_ = aDouble; }
    

    /**
     *  Draw the axes so that y=0 is included
     */
    private boolean includeYZero_;
    public boolean includeYZero() { return includeYZero_; }
    public void includeYZero( boolean aBoolean ) { includeYZero_ = aBoolean; }
    
    /**
     *  Draw the axes so that x=0 is included
     */
    private boolean includeXZero_;
    public boolean includeXZero() { return includeXZero_; }
    public void includeXZero( boolean aBoolean ) { includeXZero_ = aBoolean; }

    /**
     *  The starting value of maxX so that the graph has
     *  a reasonable starting appearance
     */
    private double initialMaxX_;
    public double initialMaxX() { return initialMaxX_; }
    public void initialMaxX( double aDouble ) { initialMaxX_ = aDouble; }
    
    /**
     *  The starting value of maxY so that the graph has
     *  a reasonable starting appearance
     */
    private double initialMaxY_;
    public double initialMaxY() { return initialMaxY_; }
    public void initialMaxY( double aDouble ) { initialMaxY_ = aDouble; }


    public Graph(Vector dataSeries, double initialMaxX, double initialMaxY,
                                           boolean includeXZero, boolean includeYZero) {
        this();
        this.dataSeries(dataSeries);
        this.includeXZero(includeXZero);
        this.includeYZero(includeYZero);
        this.initialMaxX(initialMaxX);
        this.initialMaxY(initialMaxY);
        this.findExtrema();
    }
    
 //   public Graph(String registryName) {
 //       this();
 //       ObservableRegistry.instance().register(registryName, this);
 //   }
            
    public Graph() {
        this.initialize();
    }

    private void initialize() {
        this.dataSeries(new Vector());
        this.minX(0.0);
        this.maxX(1.0);
        this.minY(0.0);
        this.maxY(1.0);
        
        this.includeXZero(false);
        this.includeYZero(false);
        this.initialMaxX(100);
        this.initialMaxY(100);
        
        
        System.out.println("init Graph");
        ObservableRegistry.instance().register("graph", this);
    }

    public void publicize() {
        this.setChanged();
        this.notifyObservers();
    }
    
    public Enumeration elements() {
        return this.dataSeries().elements();
    }
    
    public void addPoint(double xValue, double yValue) {
        this.dataSeries().addElement(new RealNumberPoint(xValue, yValue));
        this.findExtrema();
    }
    
    public void reset() {
        this.dataSeries(new Vector());
    }

    public void plot() {
        
        //System.out.println("inside plot");

        // fake abscissa if necessary ,, maybe later

        this.publicize(); // tell the GraphCanvas we are done
    }
    
    
    public double forcedMinX() {
        double m = this.minX();
        if ((this.includeXZero()) && (m > 0.0)) {
            return 0.0;
        }
        return m;
    }
    
    public double forcedMinY() {
        double m = this.minY();
        if ((this.includeYZero()) && (m > 0.0)) {
            return 0.0;
        }
        return m;
    }
    
    public double forcedMaxX() {
        double m = this.maxX();
        if (this.initialMaxX() > m) {
            m = this.initialMaxX();
        }
        if ((this.includeXZero()) && (m < 0.0)) {
            return 0.0;
        }
        return m;
    }
    
    public double forcedMaxY() {
        double m = this.maxY();
        if (this.initialMaxY() > m) {
            m = this.initialMaxY();
        }
        if ((this.includeYZero()) && (m < 0.0)) {
            return 0.0;
        }
        return m;
    }

    
    
    
    
    public void findExtrema() {

        double theMinX = Double.POSITIVE_INFINITY;
        double theMinY = Double.POSITIVE_INFINITY;
        double theMaxX = Double.NEGATIVE_INFINITY;
        double theMaxY = Double.NEGATIVE_INFINITY;
        
        Enumeration theOC = this.dataSeries().elements();
        while (theOC.hasMoreElements()) {
            RealNumberPoint thePoint = (RealNumberPoint) theOC.nextElement();
            if (thePoint.x < theMinX) {
                theMinX = thePoint.x;
            }
            if (thePoint.y < theMinY) {
                theMinY = thePoint.y;
            }
            if (thePoint.x > theMaxX) {
                theMaxX = thePoint.x;
            }
            if (thePoint.y > theMaxY) {
                theMaxY = thePoint.y;
            }
        }
        
        // tell everybody
        
        this.minX(theMinX);
        this.minY(theMinY);
        this.maxX(theMaxX);
        this.maxY(theMaxY);
    }
    
    
}