package graph;

import java.awt.*;
import java.util.*;
import java.text.*;
import java.lang.*;
import java.io.*;
import java.awt.image.*;

public abstract class GraphCanvas extends Canvas implements Observer {

    private static final int GRIDCOLOR  = 0xff999999;
    private static final int BACKCOLOR  = 0xffffffff;
    private static final int PADCOLOR   = 0xff005555;
    private static final int TITLECOLOR = 0xffaa5500;

    // the graph associated with this canvas
    private Graph graph_;
    public Graph graph() { return graph_; }
    public void graph(Graph aGraph) { graph_ = aGraph; }

    // the size of each element in pixels
    private Point elementSize_;
    public Point elementSize() { return elementSize_ ; }
    public void elementSize(Point theSize) { elementSize_ = theSize ; }

    // the insets in pixels of the actual graph in the view
    private Insets graphInsets_;
    public Insets graphInsets() { return graphInsets_ ; }
    public void graphInsets(Insets theInsets) { graphInsets_ = theInsets ; }
    
    // the graph title
    private String title_;
    public String title() { return title_; }
    public void title(String aTitle) { title_ = aTitle; }
    
    protected GraphCanvas() {
        this.setSize(100,100);
        this.setLocation(100,199);
        this.setForeground(Color.green);
        this.setBackground(Color.black);
        this.graph( null );
        this.initialize();
    }

    public GraphCanvas(String title) {
        this();
        this.title(title);
    }

    private void initialize() {
        this.title("Graph");
        System.out.println("init GraphView");
    }

    public void initializeGraphics() {
        this.determineScaling();
    }

    public void hookupObservers() {
   //     this.graph().addObserver(this);
    }

    public void determineScaling() {
        
        int newWidth =  (int)  (0.65 * this.getSize().width);
        int newHeight = (int)  (0.65 * this.getSize().height);

        // center it

        int xDelta = (this.getSize().width  - newWidth)  / 2;
        int yDelta = (this.getSize().height - newHeight) / 2;

        this.graphInsets(new Insets(yDelta, xDelta,
                          yDelta+newHeight, xDelta+newWidth));
        System.out.println("Insets: top = " + this.graphInsets().top +
                               " bottom = " + this.graphInsets().bottom +
                                 " left = " + this.graphInsets().left +
                                " right = " + this.graphInsets().right);
    }

    public void drawTitle(Graphics g) {

        if(this.graphInsets()==null) return;

        g.setColor(new Color(GraphCanvas.TITLECOLOR));

        Font theFont = new Font("sansserif", Font.PLAIN, 14);
//        FontMetrics theMetrics = Toolkit.getDefaultToolkit().getFontMetrics(theFont);
        FontMetrics theMetrics = this.getFontMetrics(theFont);
        int titleWidth = theMetrics.stringWidth(this.title());

        int xPos = this.graphInsets().left + (this.graphInsets().right -
                                 this.graphInsets().left - titleWidth) / 2;
        int yPos = this.graphInsets().top - 1;

        g.setFont(theFont);

        g.drawString(this.title(), xPos, yPos);
    }



    public void drawFrame(Graphics g) {

        g.setColor(new Color(GraphCanvas.GRIDCOLOR));
        if (this.graphInsets() == null) {return; }
        g.drawLine(this.graphInsets().left, this.graphInsets().top,
                      this.graphInsets().left, this.graphInsets().bottom);
        g.drawLine(this.graphInsets().left, this.graphInsets().bottom,
                      this.graphInsets().right, this.graphInsets().bottom);

        // axis labels
        
        NumberFormat theNF = new DecimalFormat("#########0.00");
        
        // skip them if there is no graph yet
        if (this.graph() != null) {

            g.drawString(theNF.format(this.graph().forcedMaxY()), 10,                       this.graphInsets().top);
            g.drawString(theNF.format(this.graph().forcedMinY()), 10,                       this.graphInsets().bottom);
            g.drawString(theNF.format(this.graph().forcedMinX()), this.graphInsets().left,  this.graphInsets().bottom+20);
            g.drawString(theNF.format(this.graph().forcedMaxX()), this.graphInsets().right, this.graphInsets().bottom+20);
        }
    }

    public void getHorizontalTicks() {
    }

    public void getVerticalTicks() {
    }


    public void update(Observable obs, Object o) {
        //System.out.println("inside GraphCanvas update");
        this.graph((Graph) obs);
        this.repaint();
    }

    public final void drawState(int anX, int aY) {
    }

    // public void update(Graphics g) { this.paint(g); }  // fixbg .. could this be used?

    public void paint (Graphics g) {
        this.drawFrame(g);
        this.drawTitle(g);
        // skip the graph is there is no graph object yet
        if (this.graph() != null) {
            this.drawGraph(g);
        }
    }

    public abstract void drawGraph(Graphics g);

}