package graph;

import java.util.*;

public class Histogram extends Observable {
    
    /**
     *  This is where the raw data is stored.
     *  All statistics are generated using this input data
     */
    private Vector inputData_;
    public Vector inputData() { return inputData_; }
    public void inputData( Vector aVector ) { inputData_ = aVector; }
    
    /**
     *  The minimum input data element
     */
    private double inputDataMin_;
    public double inputDataMin() { return inputDataMin_; }
    public void inputDataMin( double aDouble ) { inputDataMin_ = aDouble; }

    /**
     *  The minimum input data element
     */
    private double inputDataMax_;
    public double inputDataMax() { return inputDataMax_; }
    public void inputDataMax( double aDouble ) { inputDataMax_ = aDouble;}
    
    
    // -- Bin Data IVs --

    /**
     *  The bin data associated generated from
     *  the input data
     */
    private Vector binData_;
    public Vector binData() { return binData_; }
    protected void binData(Vector theData) { binData_ = theData; }

    /**
     *  Draw the axes so that y=0 is included
     */
    private boolean includeBinYZero_;
    public boolean includeBinYZero() { return includeBinYZero_; }
    public void includeBinYZero( boolean aBoolean ) { includeBinYZero_ = aBoolean; }
    
    /**
     *  Draw the axes so that x=0 is included
     */
    private boolean includeBinXZero_;
    public boolean includeBinXZero() { return includeBinXZero_; }
    public void includeBinXZero( boolean aBoolean ) { includeBinXZero_ = aBoolean; }

    /**
     *  The starting value of maxX so that the graph has
     *  a reasonable starting appearance
     */
    private double initialBinDataMaxX_;
    public double initialBinDataMaxX() { return initialBinDataMaxX_; }
    public void initialBinDataMaxX( double aDouble ) { initialBinDataMaxX_ = aDouble; }
    
    /**
     *  The starting value of maxY so that the graph has
     *  a reasonable starting appearance
     */
    private double initialBinDataMaxY_;
    public double initialBinDataMaxY() { return initialBinDataMaxY_; }
    public void initialBinDataMaxY( double aDouble ) { initialBinDataMaxY_ = aDouble; }

    private double binDataMinX_;
    public double binDataMinX() { return binDataMinX_; }
    public void binDataMinX(double aDouble) { binDataMinX_ = aDouble; }

    private double binDataMaxX_;
    public double binDataMaxX() { return binDataMaxX_; }
    public void binDataMaxX(double aDouble) { binDataMaxX_ = aDouble; }

    private double binDataMinY_;
    public double binDataMinY() { return binDataMinY_; }
    public void binDataMinY(double aDouble) { binDataMinY_ = aDouble; }

    private double binDataMaxY_;
    public double binDataMaxY() { return binDataMaxY_; }
    public void binDataMaxY(double aDouble) { binDataMaxY_ = aDouble; }
    

// -- Input Data Statistics -- 

    /**
     *  Contains the frequencies of elements in the
     *  input data
     */
    private Vector frequencies_;
    public Vector frequencies() { return frequencies_; }
    public void frequencies( Vector aVector ) { frequencies_ = aVector; }

    /**
     *  The sum of the input data
     */
    private double elementSum_;
    public double elementSum() { return elementSum_; }
    public void elementSum( double aDouble ) { elementSum_ = aDouble; }

    /**
     *  The sum of the input data element squares
     */
    private double elementSquaresSum_;
    public double elementSquaresSum() { return elementSquaresSum_; }
    public void elementSquaresSum( double aDouble ) { elementSquaresSum_ = aDouble; }
   
    /**
     *  The number of historgram bins
     */
    private int numberOfBins_;
    public int numberOfBins() { return numberOfBins_; }
    public void numberOfBins( int anInt ) { numberOfBins_ = anInt; }


    /**
     *  The range of each bin
     */
    private double binDelta_;
    public double binDelta() { return binDelta_; }
    public void binDelta( double aDouble ) { binDelta_ = aDouble; }

    /**
     *  The mean of the input data
     */
    private double mean_;
    public double mean() { return mean_; }
    public void mean( double aDouble ) { mean_ = aDouble; }

    /**
     *  The variance of the input data
     */
    private double variance_;
    public double variance() { return variance_; }
    public void variance( double aDouble ) { variance_ = aDouble; }

    /**
     *  The standard deviation of the input data
     */
    private double standardDeviation_;
    public double standardDeviation() { return standardDeviation_; }
    public void standardDeviation( double aDouble ) { standardDeviation_ = aDouble; }

    /**
     *  The area associated with this histogram
     */
    private double area_;
    public double area() { return area_; }
    public void area( double aDouble ) { area_ = aDouble; }


    public Histogram (int numberOfBins, double initialBinDataMaxX, double initialBinDataMaxY,
                                              boolean includeBinXZero, boolean includeBinYZero) {

        this();
        this.numberOfBins(numberOfBins);
        this.includeBinXZero(includeBinXZero);
        this.includeBinYZero(includeBinYZero);
        this.initialBinDataMaxX(initialBinDataMaxX);
        this.initialBinDataMaxY(initialBinDataMaxY);
    }
    
    public Histogram() {
        this.initialize();
    }
    
    private void initialize() {
               
        this.frequencies(new Vector());
        this.elementSum(0.0);
        this.elementSquaresSum(0.0);
        this.inputDataMin(0.0);
        this.inputDataMax(0.0);
        this.numberOfBins(20);
        this.inputData(new Vector());
        this.binDelta(0.0);
        this.mean(0.0);
        this.variance(0.0);
        this.standardDeviation(0.0);
        this.area(0.0);
        //this.initializeTheFrequencies();
        
        this.binData(new Vector());
    
        this.includeBinXZero(true);
        this.includeBinYZero(true);
        this.initialBinDataMaxX(10);
        this.initialBinDataMaxY(10);
        
        this.binDataMinX(0.0);
        this.binDataMinY(0.0);
        this.binDataMaxX(0.0);
        this.binDataMaxY(0.0);
    }
    
    public Enumeration elements() {
        return this.binData().elements();
    }
        
    public void addDataValue(double aPoint) {
        this.inputData().addElement(new Double(aPoint));
    }
           
    public void reset() {
        this.inputData(new Vector());
        //this.initializeTheFrequencies();
    }
    
  //  public double binCountStandardDeviation() {
  //      return this.binCountStandardDeviation();
  //  }
    
    public synchronized void plot() {
        // process the inputData and put into the binData
        this.binData(this.obtainFrequencies());
        
        // set the binData mins and maxs
        this.findBinDataExtrema();
        // fake abscissa if necessary ,, maybe later
        this.publicize(); // tell the Canvas we are done
    }
    
    public void findBinDataExtrema() {

        double theMinX = Double.POSITIVE_INFINITY;
        double theMinY = Double.POSITIVE_INFINITY;
        double theMaxX = Double.NEGATIVE_INFINITY;
        double theMaxY = Double.NEGATIVE_INFINITY;
        
        Enumeration theOC = this.binData().elements();
        while (theOC.hasMoreElements()) {
            RealNumberPoint thePoint = (RealNumberPoint) theOC.nextElement();
            if (thePoint.x < theMinX) {
                theMinX = thePoint.x;
            }
            if (thePoint.y < theMinY) {
                theMinY = thePoint.y;
            }
            if (thePoint.x > theMaxX) {
                theMaxX = thePoint.x;
            }
            if (thePoint.y > theMaxY) {
                theMaxY = thePoint.y;
            }
        }
        
        // tell everybody
        
        this.binDataMinX(theMinX);
        this.binDataMinY(theMinY);
        this.binDataMaxX(theMaxX);
        this.binDataMaxY(theMaxY);
    }
            
    public Vector obtainFrequencies() {
        
        //this.initializeTheFrequencies();
        this.calculateInputDataStatistics();
        this.binTheInput();
        this.computeArea();
        this.computeRemainingInputDataStatistics();
        
        return this.frequencies();  // fixbg .. allow choice at creation
        //return this.logOfFrequencies();
    }
    
    protected Vector logOfFrequencies() {
        Vector logF = new Vector();
        Enumeration theOC = this.frequencies().elements();
        while(theOC.hasMoreElements()) {
            RealNumberPoint aPoint = (RealNumberPoint) theOC.nextElement();
            double theLogF = Math.log(aPoint.y + 1);
            logF.addElement(new RealNumberPoint(aPoint.x, theLogF));
            //System.out.println("-----aPoint.x " + aPoint.x);
        }
        return logF;
    }
    
/*        
    public void initializeTheFrequencies() {
        this.frequencies(new Vector());
        for (int i=0; i<this.numberOfBins();i++) {
            this.frequencies().addElement(new RealNumberPoint(i, 0.0));
        }
    }
*/
    
    protected void calculateInputDataStatistics() {
        
        double theMin = Double.POSITIVE_INFINITY;
        double theMax = Double.NEGATIVE_INFINITY;
        
        double theSum = 0.0;
        double theSumOfSquares = 0.0;
        
        // calc the min, max, sum and sum of squares
        
        Enumeration theOC = this.inputData().elements();
        while (theOC.hasMoreElements()) {
            double theValue = ((Double) theOC.nextElement()).doubleValue();
            if (theValue > theMax) {
                theMax = theValue;
            }
            if (theValue < theMin) {
                theMin = theValue;
            }
            theSum += theValue;
            theSumOfSquares += theValue * theValue;
        }
        this.elementSum(theSum);
        this.elementSquaresSum(theSumOfSquares);
        this.inputDataMin(theMin);
        this.inputDataMax(theMax);
    }
            
    protected void binTheInput() {
        
        double binDelta = (this.inputDataMax() - this.inputDataMin() ) / this.numberOfBins();
        if (binDelta == 0.0) binDelta = 1.0e-20; // keep us from blowing up
        this.binDelta(binDelta);
        
        // initialize the bins
        this.frequencies(new Vector());
        for (int i=0; i<this.numberOfBins();i++) {
            double binCenterValue = ((double) i * this.binDelta()) + 0.5 * this.binDelta() + this.inputDataMin();
            this.frequencies().addElement(new RealNumberPoint(binCenterValue, 0.0));
        }
        
        // do the binning
        Enumeration theOC = this.inputData().elements();
        while (theOC.hasMoreElements()) {
            double anInputValue = ((Double) theOC.nextElement()).doubleValue();
            int binToIncrement = (int) (((anInputValue - this.inputDataMin()) / (1.0000001 * this.binDelta())));
            double oldFrequency = ((RealNumberPoint) this.frequencies().elementAt(binToIncrement)).y;
            RealNumberPoint theRealNumberPoint = (RealNumberPoint) this.frequencies().elementAt(binToIncrement);
            theRealNumberPoint.y = oldFrequency + 1.0;
        }
    }
    
    protected void computeArea() {
        // compute the area .. used in the calculation of the normal plot overlay
        
        double theArea = 0.0;
        
        Enumeration theOC = this.frequencies().elements();
        while (theOC.hasMoreElements()) {
            double aFrequency = ((RealNumberPoint) theOC.nextElement()).y;
            theArea += aFrequency * this.binDelta(); 
        }
        this.area(theArea);
    }
    
    protected void computeRemainingInputDataStatistics() {
        double a = (double) this.inputData().size();
        
        double theMean = this.elementSum() / a; 
        double b = this.elementSquaresSum() - (this.elementSum() * this.elementSum()) / a;
        double theVariance = b / (a - 1);
        double standardDeviation = Math.sqrt(theVariance);
       // double barValue = (this.max() - this.min()) / 10.0; // fixbg .. what the heck is this
        this.mean(theMean);
        this.variance(theVariance);
        this.standardDeviation(standardDeviation);
        //System.out.println("variance inside hdc = " + theVariance);
    }
    
    public void publicize() {
        this.setChanged();
        this.notifyObservers();
    }
        
    
    public void displayFrequencies() {
        Enumeration theOC = this.frequencies().elements();
        while(theOC.hasMoreElements()) {
            RealNumberPoint aPoint = (RealNumberPoint) theOC.nextElement();
            //System.out.println("freq = " + aPoint.y);
        }
    }
    
    public void displayInput() {
        Enumeration theOC = this.inputData().elements();
        while(theOC.hasMoreElements()) {
            Double aValue = (Double) theOC.nextElement();
            //System.out.println("input = " + aValue);
        }
    }
   
    
  /*  
    public double binCountStandardDeviation() {
        
        double sum = 0.0;
        double sumSquared = 0.0;
        double numberOfFrequencies = (double) this.frequencies().size();
        if (numberOfFrequencies < 2) return 0.0;
        
        this.calculateAndConvert();
        
        Enumeration theOC = this.frequencies().elements();
        while(theOC.hasMoreElements()) {
            RealNumberPoint theFrequency = (RealNumberPoint) theOC.nextElement();
            double theCount = theFrequency.y;
            sum += theCount;
            sumSquared += theCount * theCount;
        }
        
        double b = sumSquared - ((sum * sum) / numberOfFrequencies);
        double binCountVariance = b / (numberOfFrequencies - 1);
        
        return Math.sqrt(binCountVariance);
    }
    */
    
    /*
    
    public double minX() {
        double m = this.graph().minX();
        if ((this.includeXZero()) && (m > 0.0)) {
            return 0.0;
        }
        return m;
    }
    
    public double minY() {
        double m = this.graph().minY();
        if ((this.includeYZero()) && (m > 0.0)) {
            return 0.0;
        }
        return m;
    }
    
    public double maxX() {
        double m = this.graph().maxX();
        if (this.initialMaxX() > m) {
            m = this.initialMaxX();
        }
        if ((this.includeXZero()) && (m < 0.0)) {
            return 0.0;
        }
        return m;
    }
    
    public double maxY() {
        double m = this.graph().maxY();
        if (this.initialMaxY() > m) {
            m = this.initialMaxY();
        }
        if ((this.includeYZero()) && (m < 0.0)) {
            return 0.0;
        }
        return m;
    }
    
   */ 
    
}
        
