package graph;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.text.*;

public class HistogramCanvas  extends Canvas implements Observer {
    
    private static final int GRIDCOLOR  = 0xff999999;
    private static final int BACKCOLOR  = 0xffffffff;
    private static final int PADCOLOR   = 0xff005555;
    private static final int TITLECOLOR = 0xffaa5500;
    
    // the histogram associated with this canvas
    private Histogram histogram_;
    public Histogram histogram() { return histogram_; }
    public void histogram(Histogram aHistogram) { histogram_ = aHistogram; }

    // the insets in pixels of the actual histogram in the view
    private Insets graphInsets_;
    public Insets graphInsets() { return graphInsets_ ; }
    public void graphInsets(Insets theInsets) { graphInsets_ = theInsets ; }
    
    // the histogram title
    private String title_;
    public String title() { return title_; }
    public void title(String aTitle) { title_ = aTitle; }
       
    public HistogramCanvas() {
        this.initialize();
    }
    
    public HistogramCanvas(String title) {
        this();
        this.title(title);
   }
    
    private void initialize() {
        this.setForeground(Color.green);
        this.setBackground(Color.black);
    }
    
    /*
    public void update(Observable obs, Object o) {
        //System.out.println("inside GraphCanvas update");
        this.graph((Histogram) obs);
        this.repaint();
    }
    */
    
    public void initializeGraphics() {
        this.determineScaling();
    }

    
    public void drawFrame(Graphics g) {

        g.setColor(new Color(HistogramCanvas.GRIDCOLOR));
        if (this.graphInsets() == null) {return; }
        g.drawLine(this.graphInsets().left, this.graphInsets().top,
                      this.graphInsets().left, this.graphInsets().bottom);
        g.drawLine(this.graphInsets().left, this.graphInsets().bottom,
                      this.graphInsets().right, this.graphInsets().bottom);

        // axis labels
        
        NumberFormat theNF = new DecimalFormat("#########0.00");
        
        // skip them if there is no graph yet
        if (this.histogram() != null) {

            g.drawString(theNF.format(this.histogram().binDataMaxY()), 10, this.graphInsets().top);
            g.drawString(theNF.format(this.histogram().binDataMinY()), 10, this.graphInsets().bottom);
            
            // why was this there before 
            //g.drawString(theNF.format(((Histogram)this.graph()).actualMinX()), this.graphInsets().left,  this.graphInsets().bottom+20);
            //g.drawString(theNF.format(((Histogram)this.graph()).actualMaxX()), this.graphInsets().right, this.graphInsets().bottom+20);
            
            g.drawString(theNF.format(this.histogram().inputDataMin()), this.graphInsets().left,  this.graphInsets().bottom+20);
            g.drawString(theNF.format(this.histogram().inputDataMax()), this.graphInsets().right, this.graphInsets().bottom+20);
        }
    }    
    
    public void drawGraph(Graphics g) {
        if (this.graphInsets() == null) {return; }

        g.setColor(new Color(HistogramCanvas.PADCOLOR));
        int xDelta = this.graphInsets().right  - this.graphInsets().left;
        int yDelta = this.graphInsets().bottom - this.graphInsets().top;

        Point base = new Point(this.graphInsets().left,
                                           this.graphInsets().bottom);
        double xMin = this.histogram().binDataMinX();
        double yMin = this.histogram().binDataMinY();
        double xMax = this.histogram().binDataMaxX();
        //double xMax = 150.0;
        double yMax = this.histogram().binDataMaxY();
        
        //System.out.println("minX " + xMin + ", maxX = " + xMax);

        double xRange = (double) (xMax - xMin);
        double yRange = (double) (yMax - yMin);

        double xScale = xDelta / xRange;
        double yScale = yDelta / yRange;

        // now let's prepare a little more

        Enumeration theData = this.histogram().elements();
        RealNumberPoint aPoint = null;
        double xP = 0.0;
        double yP = 0.0;
        int xPixels = 0;
        int yPixels = 0;
          
        // shift it so the bar is centered at x,y ??
        
        // draw everybody
        //System.out.println("inside HistogramCanvas .. x pos to plot ..");
        while(theData.hasMoreElements()) {
            aPoint = (RealNumberPoint) theData.nextElement();
            //System.out.print(" " + aPoint.x + " ");
            xP = (double) aPoint.x - xMin;
            yP = (double) aPoint.y - yMin;
            xPixels = (int) (xP * xScale);
            yPixels = (int) (yP * yScale);
            //g.fillRect ((base.x + xPixels), (base.y - yPixels), 2, yPixels);
            g.fillRect ((base.x + xPixels), (base.y - yPixels - 1), 2, yPixels);
        }

    }
    
    public void drawTitle(Graphics g) {

        if(this.graphInsets()==null) return;

        g.setColor(new Color(HistogramCanvas.TITLECOLOR));

        Font theFont = new Font("sansserif", Font.PLAIN, 14);
//        FontMetrics theMetrics = Toolkit.getDefaultToolkit().getFontMetrics(theFont);
        FontMetrics theMetrics = this.getFontMetrics(theFont);
        int titleWidth = theMetrics.stringWidth(this.title());

        int xPos = this.graphInsets().left + (this.graphInsets().right -
                                 this.graphInsets().left - titleWidth) / 2;
        int yPos = this.graphInsets().top - 1;

        g.setFont(theFont);

        g.drawString(this.title(), xPos, yPos);
    }
    
    public void determineScaling() {
        
        int newWidth =  (int)  (0.65 * this.getSize().width);
        int newHeight = (int)  (0.65 * this.getSize().height);

        // center it

        int xDelta = (this.getSize().width  - newWidth)  / 2;
        int yDelta = (this.getSize().height - newHeight) / 2;

        this.graphInsets(new Insets(yDelta, xDelta,
                          yDelta+newHeight, xDelta+newWidth));
        //System.out.println("Insets: top = " + this.graphInsets().top +
                             //  " bottom = " + this.graphInsets().bottom +
                             //    " left = " + this.graphInsets().left +
                             //   " right = " + this.graphInsets().right);
    }

    
    public void update(Observable obs, Object o) {
        //System.out.println("inside GraphCanvas update");
        this.histogram((Histogram) obs);
        this.repaint();
    }


    // public void update(Graphics g) { this.paint(g); }  // fixbg .. could this be used?

    public void paint (Graphics g) {
        this.drawFrame(g);
        this.drawTitle(g);
        // skip the graph is there is no graph object yet
        if (this.histogram() != null) {
            this.drawGraph(g);
        }
    }

}