package graph;

import java.util.*;

public class HistogramDataConverter {

    /**
     *  Contains the frequencies of elements in the
     *  input data
     */
    private Vector frequencies_;
    public Vector frequencies() { return frequencies_; }
    public void frequencies( Vector aVector ) { frequencies_ = aVector; }

    /**
     *  The sum of the input data
     */
    private double elementSum_;
    public double elementSum() { return elementSum_; }
    public void elementSum( double aDouble ) { elementSum_ = aDouble; }

    /**
     *  The sum of the input data element squares
     */
    private double elementSquaresSum_;
    public double elementSquaresSum() { return elementSquaresSum_; }
    public void elementSquaresSum( double aDouble ) { elementSquaresSum_ = aDouble; }

    /**
     *  The minimum input data element
     */
    private double min_;
    public double min() { return min_; }
    public void min( double aDouble ) { min_ = aDouble; }

    /**
     *  The minimum input data element
     */
    private double max_;
    public double max() { return max_; }
    public void max( double aDouble ) { max_ = aDouble; }
    
    /**
     *  The number of historgram bins
     */
    private int numberOfBins_;
    public int numberOfBins() { return numberOfBins_; }
    public void numberOfBins( int anInt ) { numberOfBins_ = anInt; }

    /**
     *  All statistics are generated using this input data
     */
    private Vector inputData_;
    public Vector inputData() { return inputData_; }
    public void inputData( Vector aVector ) { inputData_ = aVector; }

    /**
     *  The range of each bin
     */
    private double binDelta_;
    public double binDelta() { return binDelta_; }
    public void binDelta( double aDouble ) { binDelta_ = aDouble; }

    /**
     *  The mean of the input data
     */
    private double mean_;
    public double mean() { return mean_; }
    public void mean( double aDouble ) { mean_ = aDouble; }

    /**
     *  The variance of the input data
     */
    private double variance_;
    public double variance() { return variance_; }
    public void variance( double aDouble ) { variance_ = aDouble; }

    /**
     *  The standard deviation of the input data
     */
    private double standardDeviation_;
    public double standardDeviation() { return standardDeviation_; }
    public void standardDeviation( double aDouble ) { standardDeviation_ = aDouble; }

    /**
     *  The area associated with this histogram
     */
    private double area_;
    public double area() { return area_; }
    public void area( double aDouble ) { area_ = aDouble; }
/*    
    public HistogramDataConverter(Vector inputElements, int numberOfBins) {
        this();
        this.inputData(inputElements);
        this.numberOfBins(numberOfBins);
        this.calculateAndConvert();
    }
*/

    public HistogramDataConverter(int numberOfBins) {
        this();
        this.numberOfBins(numberOfBins);
    }
    
    public HistogramDataConverter() {
        this.initialize();
    }
    
    private void initialize() {
        this.frequencies(new Vector());
        this.elementSum(0.0);
        this.elementSquaresSum(0.0);
        this.min(0.0);
        this.max(0.0);
        this.numberOfBins(20);
        this.inputData(new Vector());
        this.binDelta(0.0);
        this.mean(0.0);
        this.variance(0.0);
        this.standardDeviation(0.0);
        this.area(0.0);
        this.initializeTheFrequencies();
    }
    
    public void addPoint(RealNumberPoint aPoint) {
        this.inputData().addElement(aPoint);
    }
    
    public void reset() {
        this.inputData(new Vector());
        this.initializeTheFrequencies();
    }
    
    public void displayFrequencies() {
        Enumeration theOC = this.frequencies().elements();
        while(theOC.hasMoreElements()) {
            RealNumberPoint aPoint = (RealNumberPoint) theOC.nextElement();
            System.out.println("freq = " + aPoint.y);
        }
    }
    
    public void displayInput() {
        Enumeration theOC = this.inputData().elements();
        while(theOC.hasMoreElements()) {
            RealNumberPoint aPoint = (RealNumberPoint) theOC.nextElement();
            System.out.println("input = " + aPoint.y);
        }
    }
   
    public Vector obtainFrequenciesGiven(double min, double max) {
        this.calculateAndConvertUsing(min, max);
       // return this.frequencies();  // fixbg .. allow choice at creation
        return this.logOfFrequencies();
    }
    
    protected Vector logOfFrequencies() {
        Vector logF = new Vector();
        Enumeration theOC = this.frequencies().elements();
        while(theOC.hasMoreElements()) {
            RealNumberPoint aPoint = (RealNumberPoint) theOC.nextElement();
            double theLogF = Math.log(aPoint.y + 1);
            logF.addElement(new RealNumberPoint(aPoint.x, theLogF));
        }
        return logF;
    }
    
    protected void calculateAndConvertUsing(double min, double max) {
        this.initializeTheFrequencies();
        this.calculateSimpleStatistics();
        this.binTheInput(min, max);
        this.computeArea();
        this.computeRemainingStatistics();
    }
    
    public void initializeTheFrequencies() {
        this.frequencies(new Vector());
        for (int i=0; i<this.numberOfBins();i++) {
            this.frequencies().addElement(new RealNumberPoint(i, 0.0));
        }
    }
    
    protected void calculateSimpleStatistics() {
        
        double theMin = Double.POSITIVE_INFINITY;
        double theMax = Double.NEGATIVE_INFINITY;
        
        double theSum = 0.0;
        double theSumOfSquares = 0.0;
        
        // calc the min, max, sum and sum of squares
        
        Enumeration theOC = this.inputData().elements();
        while (theOC.hasMoreElements()) {
            double theValue = ((RealNumberPoint) theOC.nextElement()).y;
            if (theValue > theMax) {
                theMax = theValue;
            }
            if (theValue < theMin) {
                theMin = theValue;
            }
            theSum += theValue;
            theSumOfSquares += theValue * theValue;
        }
        this.elementSum(theSum);
        this.elementSquaresSum(theSumOfSquares);
        this.min(theMin);
        this.max(theMax);
        System.out.println("min= "+this.min() + "  max= " + this.max());
    }
            
    protected void binTheInput(double min, double max) {
        
        double binDelta = (this.max() - this.min() ) / this.numberOfBins();
        if (binDelta == 0.0) binDelta = 1.0; // keep us from blowing up
        this.binDelta(binDelta);

        // do the binning
        Enumeration theOC = this.inputData().elements();
        while (theOC.hasMoreElements()) {
            double anInputValue = ((RealNumberPoint) theOC.nextElement()).y;
           // System.out.println("input value = " + anInputValue + " bindelta = " + this.binDelta());
            int binToIncrement = (int) (((anInputValue - this.min()) / (1.0000001 * this.binDelta())));
            double oldFrequency = ((RealNumberPoint) this.frequencies().elementAt(binToIncrement)).y;
            double newFrequency = oldFrequency + 1.0;
            this.frequencies().setElementAt (new RealNumberPoint(binToIncrement, newFrequency), binToIncrement);
        }
    }
    
    protected void computeArea() {
        // compute the area .. used in the calculation of the normal plot overlay
        
        double theArea = 0.0;
        
        Enumeration theOC = this.frequencies().elements();
        while (theOC.hasMoreElements()) {
            double aFrequency = ((RealNumberPoint) theOC.nextElement()).y;
            theArea += aFrequency * this.binDelta(); 
        }
        this.area(theArea);
    }
    
    protected void computeRemainingStatistics() {
        double a = (double) this.inputData().size();
        
        double theMean = this.elementSum() / a; 
        double b = this.elementSquaresSum() - (this.elementSum() * this.elementSum()) / a;
        double theVariance = b / (a - 1);
        double standardDeviation = Math.sqrt(theVariance);
        double barValue = (this.max() - this.min()) / 10.0; // fixbg .. what the heck is this
        this.mean(theMean);
        this.variance(theVariance);
        this.standardDeviation(standardDeviation);
        //System.out.println("variance inside hdc = " + theVariance);
    }
  /*  
    public double binCountStandardDeviation() {
        
        double sum = 0.0;
        double sumSquared = 0.0;
        double numberOfFrequencies = (double) this.frequencies().size();
        if (numberOfFrequencies < 2) return 0.0;
        
        this.calculateAndConvert();
        
        Enumeration theOC = this.frequencies().elements();
        while(theOC.hasMoreElements()) {
            RealNumberPoint theFrequency = (RealNumberPoint) theOC.nextElement();
            double theCount = theFrequency.y;
            sum += theCount;
            sumSquared += theCount * theCount;
        }
        
        double b = sumSquared - ((sum * sum) / numberOfFrequencies);
        double binCountVariance = b / (numberOfFrequencies - 1);
        
        return Math.sqrt(binCountVariance);
    }
    */
}
        
        