package graph;

import java.util.*;
import java.awt.*;
import java.awt.event.*;

public class LineGraphCanvas extends GraphCanvas {
    
    private static final int GRIDCOLOR  = 0xff999999;
    private static final int BACKCOLOR  = 0xffffffff;
    private static final int PADCOLOR   = 0xff005555;
    private static final int TITLECOLOR = 0xffaa5500;
    
    public LineGraphCanvas() {
        this.initialize();
    }
    
    public LineGraphCanvas (String title) {
        this.title(title);
   }
    
    private void initialize() {
    }
    
    public void drawGraph(Graphics g) {
        if (this.graphInsets() == null) {return; }

        g.setColor(new Color(LineGraphCanvas.PADCOLOR));
        int xDelta = this.graphInsets().right  - this.graphInsets().left;
        int yDelta = this.graphInsets().bottom - this.graphInsets().top;

        Point base = new Point(this.graphInsets().left,
                                           this.graphInsets().bottom);
        double xMin = this.graph().minX();
        double yMin = this.graph().minY();
        double xMax = this.graph().maxX();
        double yMax = this.graph().maxY();

        double xRange = (double) (xMax - xMin);
        double yRange = (double) (yMax - yMin);

        double xScale = xDelta / xRange;
        double yScale = yDelta / yRange;

        // now let's draw it

        Enumeration theData = this.graph().elements();
        RealNumberPoint aPoint = null;
        double xP = 0.0;
        double yP = 0.0;
        int xPixels = 0;
        int yPixels = 0;
        Point last = new Point(0,0);

        // get first point
        
        if (theData.hasMoreElements()) {
            aPoint = (RealNumberPoint) theData.nextElement();
            xP = (double) aPoint.x - xMin;
            yP = (double) aPoint.y - yMin;
            xPixels = (int) (xP * xScale);
            yPixels = (int) (yP * yScale);
            last = new Point(base.x+xPixels, base.y-yPixels);
        }

        // now the rest
        while(theData.hasMoreElements()) {
            aPoint = (RealNumberPoint) theData.nextElement();
            xP = (double) aPoint.x - xMin;
            yP = (double) aPoint.y - yMin;
            xPixels = (int) (xP * xScale);
            yPixels = (int) (yP * yScale);
            g.drawLine (last.x, last.y, (base.x + xPixels), (base.y - yPixels));
            last.x = base.x + xPixels;
            last.y = base.y - yPixels;
        }

    }
}