package graph;

import java.util.*;
import java.io.*;

public class SampleDataReader {

    private static final int MAX_DATA_ELEMENTS = 17000;

    /**
     *  The reader associated with this SampleDataReader
     */
    private BufferedReader reader_;
    public BufferedReader reader() { return reader_; }
    public void reader( BufferedReader aBufferedReader) { reader_ = aBufferedReader; }

    /**
     *  The filename for this SampleDataReader
     */
    private String fileName_;
    public String fileName() { return fileName_; }
    public void fileName( String aString ) { fileName_ = aString; }

    /**
     *  The data extracted from the file
     */
    private double[] data_;
    public double[] data() { return data_; }
    public void data( double[] theDoubles ) { data_ = theDoubles; }


    private SampleDataReader() {
        this.initialize();
    }

    public SampleDataReader(String fileName) {
        this.fileName(fileName);
        this.initialize();
    }

    private void initialize() {
        try {
            
            BufferedReader theReader = new BufferedReader(
                                         new InputStreamReader(
                                             new FileInputStream(this.fileName())));
            this.reader(theReader);
        }
        catch (IOException e) {
            System.out.println("IOException in initialize of SampleDataReader");
        }
    }

    public double readDataElement() throws IOException {
        String theValue = null;
        String theData = null;

        if ((theData = this.reader().readLine()) == null) {
            throw new EOFException();
        }

        StringTokenizer theST = new StringTokenizer(theData);
        try {
            theST.nextToken(); // throw first col away
            theValue = theST.nextToken(); // the good stuff
        }
        catch (NoSuchElementException e) {
            throw new IOException();
        }

        return new Double(theValue).doubleValue();
    }

    public double[] readAllElements() {

        this.data(new double[SampleDataReader.MAX_DATA_ELEMENTS]);

        int count = 0;

        try {
            while(count < SampleDataReader.MAX_DATA_ELEMENTS) {
                this.data()[count] = this.readDataElement();
                count++;
                if (count < 10) {
                    System.out.println("count = " + count + "   data = " + this.data()[count-1]);
                }
            }
            System.out.println("Too much data " + count);
        }

        catch (EOFException e) {

            // copy the data to correct sized array
            double[] actualData = new double[count];
            for (int i=0; i<count; i++) {
                actualData[i] = this.data()[i];
            }
            return actualData;
        }
        catch (IOException e) {
            System.out.println("IOException in readAllElements in SampleDataReader");
        }

        return this.data();

    }


}